/*
   Copyright (C) 1988, 1989, 1990 The GAP Development Company

   All Rights Reserved
   
   This source code is considered to be confidential information proprietary
   to The GAP Development Company.  It is prohibited and punishable by law
   to use or duplicate any part of this information without express
   written consent from GAP Development Company.

   This source code is being provided for informational purposes only.


   UTIEXPRT.C

   UTI module that extracts forum mail that is marked as echo mail (bit
   5 of the delete flag) and places it into a UTI file.


   To compile : cl /c /FPa utiexprt.c

   To link    : link utiexprt+obj\ctclibss,,NUL.MAP,+ctreess
              : link utiexprt+obj\ctclibsm,,NUL.MAP,+ctreesm  // multi user

*/

#define LINT_ARGS

#define MULTIUSER
//#define SINGLEUSER

#pragma check_stack(off)
#pragma pack(1)

#include <stdio.h>
#include <string.h>
#include <memory.h>
#include <stdlib.h>
#include <conio.h>
#include <malloc.h>
#include <time.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <share.h>

#pragma pack()                            // cannot pack Ctree structs   
#include <ctstdr.h>
#include <ctifil.h>
#pragma pack(1)


   /*********************************************************************/
   /*                   FILE MODES FOR PARAMETER FILE                   */
   /*                                                                   */
   /*                                 Multi     Single                  */
   /*    Messages:                                                      */
   /*                Filemode1          5         4            DAT      */
   /*                Filemode2          1         0            IDX      */
   /*                                                                   */
   /*    Users:                                                         */
   /*                Filemode3          3         2            DAT      */
   /*                Filemode4          3         2            IDX      */
   /*********************************************************************/

#ifdef MULTIUSER                          // multi user file mode parameters
  #define filemode1   '5'
  #define filemode2   '1'
  #define filemode3   '3'
  #define filemode4   '3'
  #define incnum       1                  // used to increment file numbers
#else                                     // single user file mode parameters
  #define filemode1   '4'
  #define filemode2   '0'
  #define filemode3   '2'
  #define filemode4   '2'
  #define incnum       0                  // used to increment file numbers
#endif                                    // multi-user needs dummylok.dat file


   char dumpall  = 0;                     // if 1, dump all the messages 
   int datno     = 0+incnum;              // data file number to use     
   int idx4      = 4+incnum;              // index file number to use    
   int forumnum  = 0;                     // forum number to work with   
   long startmes = 0L;                    // high message # for MESSNO   

   char ourmail    [60];                  // path to forum MSGS.xxx      
   char hismail    [60];                  // path to UTI MAIL.TXT file   
   char dflt_path  [60];                  // path to GAP default dir     

   unsigned char *buffer = NULL;          // pointer to message buffer

   struct MAIL                            // structure of the header rec 
      {
      char del;                           // delete flag                 
      char to           [32];             // left to whom (indexed)      
      char from         [32];             // left by whom (indexed)      
      char sub          [26];             // subject (thread - indexed)  
      long num;                           // message number (indexed)    
      long date;                          // date left                   
      long readdate;                      // date when it was read       
      long refer;                         // refer to message number     
      long activemess;                    // number of active messages   
      char read;                          // has it been read by "to"?   
      char type;                          // private, passwrd,open, etc  
      char passwd       [13];             // password if protected       
      } mail;                             // 126 bytes 

   struct FORUM
      {
      char name         [21];             // forum name                   
      char directory    [31];             // forum dir path               
      char gen_dir      [31];             // forum gen dir path           
      char up_dir       [31];             // upload dir path              
      char private_up    [2];             // keep uploads private?        
      char keep_up       [2];             // keep uploads in forum?       
      char auto_join     [2];             // auto rejoin forum?           
      char handle        [2];             // allow handles?               
      char priv_mail     [2];             // make all mail private?       
      char pub_mail      [2];             // make all mail public?        
      char subscrip      [9];             // expired subscrip date        
      char net_mail      [2];             // is this a netmail forum?     
      char sysop1       [31];             // # 1 sysop this forum         
      char sysop2       [31];             // # 2 sysop this forum         
      char main_dirs     [2];             // include main board dirs?     
      int bullets;                        // number of bulletins          
      int dirs;                           // number of directories        
      int security;                       // additional security          
      int time;                           // additional time              
      int sys1_level;                     // level for sysop 1            
      int sys2_level;                     // level for sysop 2            
      } forum;                            // 213 bytes                    


   /**********************************************************************/
   /* A shortened version of GAPBBS.CNF.  All we need is the path to the */ 
   /* main directory and the name of the forum.dat file                  */ 
   /**********************************************************************/

   struct GAP_CONFIG
      {
      char sysname       [16];             // sysop name                   
      char syspas        [13];             // sysop local password         
      int sys_read;                        // level to read private mail   
      int sys_edit;                        // level to edit msg headers    
      int overwrite;                       // level to overwrite uploads   
      int sys_level1;                      // level to view callers log    
      int sys_level2;                      // level to view users          
      int sys_level3;                      // level to view a file         
      int sys_level4;                      // level to maintain user file  
      int sys_level5;                      // level to exit to dos         
      int sys_level5a;                     // level to execute dos funcs   
      int sys_level6;                      // level to view other nodes    
      int sys_level6a;                     // level to log off other node  
      int sys_level7;                      // level to recover messages    

      char answer         [8];             // questionnaire answers file   
      char caller         [7];             // caller file                  
      char doors          [9];             // doors.dat file               
      char forumdat       [9];             // name of forum (.DAT) file    
      char entry_news     [8];             // news file                    
      char exit_news      [8];             // exit news file               
      char pwrd           [9];             // time / security file         
      char fsec           [9];             // files security file          
      char msgs           [9];             // messages file                
      char new_welcome    [8];             // new user welcome file        
      char answer_regis   [8];             // not used                     
      char remote         [9];             // remote.sys file              
      char tcan           [9];             // trashcan file                
      char users          [9];             // users file                   
      char welcome        [8];             // welcome file                 

      char uploads        [26];            // upload directory             
      char gen            [26];            // general directory            
      char maindir        [26];            // main directory               
      char help           [26];            // help directory               
      char loose_dir      [26];            // directory where loose files are

      char dumb_modem      [2];            // use true ring detect?        
      char modem_init     [45];            // initilization string         
      char modem_off      [45];            // off hook string              
      int  modem_wait;                     // seconds to wait for carrier  
      int  connect_wait;                   // seconds to wait for connect  
      char modem_reset    [45];            // reset string                 
      char port            [2];            // com port                     
      char speed           [6];            // speed to open modem at       
      char baud_300        [2];            // restrict Baud rates?         
      char baud_start      [6];            // restrict start time          
      char baud_end        [6];            // restrict end time            
      char baud_rest      [10];            // highest baud rate to restrict
      char lock_baud       [2];            // lock in the baud rate        
      char zstream         [2];            // full flow zmodem uploads     

      int menuA;                           //  main menu security levels   
      int menuB;
      int menuC;
      int menuD;
      int menuE;
      int menuF;
      int menuG;
      int menuH;
      int menuI;
      int menuJ;
      int menuK;
      int menuL;
      int menuM;
      int menuN;
      int menuO;
      int menuP;
      int menuQ;
      int menuR;
      int menuS;
      int menuT;
      int menuU;
      int menuV;
      int menuW;
      int menuX;
      int menuY;
      int menuZ;

      int menu_chat;                      // chat with other nodes        
      int menu_open;                      // open a door                  
      int menu_who;                       // who's on other nodes         
      int regis;                          // registration level           
      int year_regis;                     // level for subscrip expired   

      int viewG;                          //  view/change stats           
      int viewN;              
      int viewL;              
      int viewT;             
      int viewX;              

      int fileA;                          //  file menu security levels   
      int fileB;                          // browse text files            
      int fileC;
      int fileD;
      int fileE;
      int fileF;                          //  file menu security levels   
      int fileG;
      int fileH;
      int fileI;
      int fileJ;
      int fileK;
      int fileL;
      int fileM;
      int fileN;
      int fileO;
      int fileP;
      int fileQ;
      int fileR;
      int fileS;
      int fileT;
      int fileU;
      int fileV;
      int fileW;
      int fileX;
      int fileY;
      int fileZ;

      int blts;                            // number of main bulletins     
      int cnfs;                            // number of FORUMS             
      int dirs;                            // number of directories        
      int printer;                         // printer port                 
      char time_limit     [2];             // enforce daily time limit     
      char byte_cnt       [2];             // enforce daily byte count     
      char private_up     [2];             // use private upload directory 
      char dir_up         [2];             // allow directory uploads      
      char closed         [2];             // closed board flag            
      char handle         [2];             // allow handles                
      char chg_name       [2];             // allow name changes           
      char regis_chk      [2];             // edit registration info       
      char subscribe      [2];             // enforce yearly registration  
      int sub_days;                        // # of days in subscrip period 
      char quick          [2];             // allow quick logons           
      char priv_mail      [2];             // make all mail private?       

      char board          [51];            // board name                   
      char comment        [51];            // comment prompt               
      char twit           [51];            // twit string with F8          
      char lockout        [51];            // lock out string with F2      
      char nu1            [51];            // future implementation        
      char nu2            [51];            // future implementation        
      int  node;                           // node number                  
      char events         [2];             // use timed events             
      char event1         [6];             // event 1 time                 
      char event2         [6];             // event 2 time                 
      int  color;                          // default color 1 - 15         
      int  free_space;                     // minimum free upload space    
      int  max_msgs;                       // maximum number of messages   
      int  maxlines;                       // maximum lines per messae     
      }config;

   /**********************************************************************/
   /*  Parameters for the ISAM mail files.  The path and name to the     */ 
   /*  message file will be filled in at run time.                       */ 
   /**********************************************************************/

#ifdef MULTIUSER
   char near dummy_file [60];                   // path to DUMMYLOK.DAT

   IFIL dummylock =
      {
      dummy_file,0,128,0,3,0,0,0,0,"delflag","delflag"
      };
#endif

   ISEG segments[] =
      {
      {1, 32,0},
      {33,32,0},
      {65,26,0},
      {91, 4,0}
      };

   IIDX indices[] =
      {
      {36,0,1,1,32,1,&segments[0],"to_idx"},
      {36,0,1,1,32,1,&segments[1],"from_idx"},
      {30,0,1,1,32,1,&segments[2],"sub_idx"},
      { 4,1,0,0, 0,1,&segments[3],"num_idx"}
      };

   IFIL imail =
      {
      ourmail,0+incnum,126,4096,filemode1,4,4096,filemode2,indices,"deleted","text"
      };

   extern int isam_err, isam_fil;

   static  int pascal write_net(void );
   static  int pascal open_gapcnf(void );
   static  int pascal open_forumdat(int which);
   static  char *trim(char *str);
   extern  void pascal run_menu(int crap);
   extern  void pascal log_call(int crap,char *crap1);


void main(argc,argv)
int argc;
char *argv[];
{
   int error = 0;

   forumnum = atoi(argv[1]);              // get the forum number to work with 
   startmes = atol(argv[2]);              // get starting message # to use     
   strcpy(hismail,argv[3]);               // path to the UTI MAIL.TXT file     

   if (argc > 3 && strcmp(argv[4],"/NETWORK") == 0)
      dumpall = 0;                        // dump only echo mail msgs   
   else
      dumpall = 1;                        // dump um all                

   if ( (error = open_gapcnf()) != 0)     // open the GAP config file   
      exit (error);

   if ( (error = open_forumdat(forumnum)) != 0)   // open forum.dat   
      exit (error);

   if (intisam(12,6,4))                   // get some memory for index buffers 
      exit(isam_err);

#ifdef MULTIUSER
   sprintf(dummy_file,"%s\\DUMMYLOK",config.maindir);

   if (opnifil(&dummylock))               // open DUMMYLOK.DAT
      exit(isam_err);
#endif

   if (opnifil(&imail))                   // open the ISAM files               
      exit(isam_err);

   error = write_net();                   // write the UTI MAIL.TXT file       

   if (error == -2)                       // -2 = cant allocate memory
      {
      cputs("\r\n\r\nCannot Allocate Memory For Message Buffer!\r\n");
      }

   clisam();                              // close the ISAM files

   exit(error);
}


   /******************************************************************/
   /*  write the network messages file.                              */ 
   /*  return 0 if successful, errno if error.                       */ 
   /******************************************************************/

static int pascal write_net()
{
   unsigned int r = 0;
   struct tm *mailtime;                   // pointer to time structure         
   
   unsigned numbytes;                     // number of bytes in ISAM message
   unsigned maxbytes;                     // max bytes allowed

   unsigned char *ptr;

   FILE *fp;

   if ( (fp = fopen(hismail,"w+t")) == NULL)   // open output file    
      return (-1);

   while ( (buffer = (unsigned char *) malloc((((config.maxlines+5)*73)+sizeof(mail)))) == NULL
            && config.maxlines > 0)
        --config.maxlines;

   if (buffer == NULL)                    // error allocating memory
      return(-2);
   
   if (config.maxlines <= 0)              // not enough lines left
      {
      free(buffer);
      return(-2);
      }

   maxbytes = (config.maxlines * 73) + sizeof(mail);

   ptr = buffer;                          // position ptr into buffer  

   if (gterec(idx4,(char *) &startmes,(char *) &mail) != 0)
      {
      fclose(fp);
      return (0);                         // there are no messages to get      
      }

   while (1)
      {
      //  message is NOT deleted and message is marked as echo mail   
      //  or message is NOT deleted and wants ALL the messages        

      if (!(mail.del & 0x80) && ((mail.del & 0x20) || dumpall))
         {
         numbytes = getvlen(datno);       // # of bytes in message

         if (numbytes <= maxbytes)        // dont overflow buffer
            {
            redvrec(datno,buffer,numbytes);  // read the whole thing

            fprintf(fp,"%s\n",mail.to);      // write the header
            fprintf(fp,"%s\n",mail.from);
            fprintf(fp,"%s\n",mail.sub);
            fprintf(fp,"%ld\n",mail.num);

            fprintf(fp,"%ld\n",mail.refer);

            mailtime = localtime(&mail.date);     // date-time of message 

            fprintf(fp,"%02d/%02d/%02d\n",mailtime->tm_mon+1,mailtime->tm_mday,
                   mailtime->tm_year);
            fprintf(fp,"%02d:%02d\n",mailtime->tm_hour,mailtime->tm_min);

            if(mail.type == 'R')
               fprintf(fp,"PRIVATE\n");   // private message
            else
               fprintf(fp,"PUBLIC\n");    // public message

            // check read status

            if ((strcmp(mail.to,"ALL")) != 0 && mail.read == 'Y')
               fprintf(fp,"Y\n");
            else
               fprintf(fp,"N\n");

            fprintf(fp,"TEXT:\n");        // flag end of header

            for (r = sizeof(mail); r < numbytes; r++)
               {
               if ( *(ptr+r) == 0)        // newline denoted by NULL
                  fprintf(fp,"\n");
               else
                  fputc(*(ptr+r),fp);
               }
                     
            fprintf(fp,"\xFF\n");         // end of message indicator 
            }
         }

      if (nxtrec(idx4,(char *) &mail))    // get next record and if no 
         break;                           // more messages, break out  
      }

   fclose(fp);                            // close his file
   free(buffer);                          // free memory
   return (0);                            // return success
}


   /******************************************************************/
   /* Open GAPBBS.CNF and read only as much of the record as we need */ 
   /******************************************************************/

static int pascal open_gapcnf()
{
   int gapfile;

   FILE *fp;

   if ( (fp = fopen("GAPUTI.CNF","rt")) == NULL)
      return (errno);

   fscanf(fp,"%s",dflt_path);             // path to GAP default directory     
   fclose(fp);

   strcat(dflt_path,"\\GAPBBS.CNF");

#ifdef MULTIUSER
   gapfile = sopen(dflt_path,O_RDONLY | O_BINARY,SH_DENYNO);
#else
   gapfile = open(dflt_path,O_RDONLY | O_BINARY);
#endif

   if(gapfile == -1)                      // cant open the file        
      return (errno);

   read(gapfile,(char *) &config,sizeof(config));
   close(gapfile);

   trim(config.maindir);                  // get rid of spaces         
   trim(config.forumdat);
   trim(config.msgs);

   return (0);
}


   /******************************************************************/
   /*  Open the Forum data file.  Get the path to the Forum default  */ 
   /*  directory where the MSGS.DAT and MSGS.IDX files are.          */ 
   /******************************************************************/

static int pascal open_forumdat(which)
int which;                                // which record to read       
{
   int forumfile;
   char forumdir  [60];

   if (which)                             // if this isnt the main board
      {
      sprintf(forumdir,"%s\\%s.DAT",config.maindir,config.forumdat);

#ifdef MULTIUSER
      forumfile = sopen(forumdir, O_RDONLY | O_BINARY,SH_DENYNO);
#else
      forumfile = open(forumdir, O_RDONLY | O_BINARY);
#endif

      if (forumfile == -1)                // cant open file            
         return (errno);

      lseek(forumfile,(long) (sizeof(forum) * (which -1)),SEEK_SET);
      read(forumfile,(char *) &forum,sizeof(forum));
      close(forumfile);

      trim(forum.directory);
      sprintf(ourmail,"%s\\%s",forum.directory,config.msgs);
      }
   else
      sprintf(ourmail,"%s\\%s",config.maindir,config.msgs);

   return(0);
}

static char *trim(str)                    // removes spaces            
char *str;
{
   char *ptr;
   
   ptr = str + strlen(str) -1;
  
   while(ptr >= str && *ptr == ' ')
      *ptr-- = 0;

   return(str);
}

   /********************************************************************/
   /*  These are two dummy functions for the ISAM routines.  If there  */
   /*  is a fatal error, the fatal is normally logged to the GAP       */
   /*  caller log and then the GAP exit routine is run.  If this       */
   /*  program ever exits with an error code of 99, then there was a   */
   /*  fatal ISAM error and the sysop will need to rebuild his ISAM    */
   /*  data files.                                                     */
   /*  They need to be declared because the ISAM routines expect them  */
   /*  to be present.                                                  */
   /********************************************************************/

void pascal run_menu(crap)
int crap;
{
   clisam();
   exit(99);
}

void pascal log_call(crap,crap1)
int crap;
char crap1[];
{
}

